if exists (select * from sysobjects where id = object_id(N'[dbo].[LockLook]') 
			and OBJECTPROPERTY(id, N'IsProcedure') = 1)
begin
	drop procedure LockLook
end
go
create procedure LockLook
@spid1 int = NULL,		/* server process id to check for locks */
@spid2 int = NULL		/* other process id to check for locks */
as
/* this procedure is adapted from sp_lock 
   it may fail if run by someone other than sa
*/
set nocount on

create table #LockLook
(spid smallint not null,
 [Database] sysname not null,
 TableID int not null,
 TableName sysname null,
 IndexID int not null,
 IndexName sysname null,
 Type char(4) not null,
 Resource char(16) not null,
 Mode char(8) not null,
 Status char(5) not null)

/*
**  Show the locks for both parameters.
*/
if @spid1 is not NULL
begin
	insert #LockLook(spid, [Database], TableID, IndexID, Type, Resource, Mode, Status)
	select 	convert (smallint, req_spid),  
		db_name(rsc_dbid), 
		rsc_objid,
		rsc_indid,
		substring (v.name, 1, 4) As Type,
		substring (rsc_text, 1, 16) as Resource,
		substring (u.name, 1, 8) As Mode,
		substring (x.name, 1, 5) As Status

	from 	master.dbo.syslockinfo,
		master.dbo.spt_values v,
		master.dbo.spt_values x,
		master.dbo.spt_values u

	where   master.dbo.syslockinfo.rsc_type = v.number
			and v.type = 'LR'
			and master.dbo.syslockinfo.req_status = x.number
			and x.type = 'LS'
			and master.dbo.syslockinfo.req_mode + 1 = u.number
			and u.type = 'L'
			
			and req_spid in (@spid1, @spid2)
end

/*
**  No parameters, so show all the locks.
*/
else
begin
	insert #LockLook(spid, [Database], TableID, IndexID, Type, Resource, Mode, Status)
	select 	convert (smallint, req_spid), 
		db_name(rsc_dbid), 
		rsc_objid,
		rsc_indid,
		substring (v.name, 1, 4) As Type,
		substring (rsc_text, 1, 16) as Resource,
		substring (u.name, 1, 8) As Mode,
		substring (x.name, 1, 5) As Status

	from 	master.dbo.syslockinfo,
		master.dbo.spt_values v,
		master.dbo.spt_values x,
		master.dbo.spt_values u

	where   master.dbo.syslockinfo.rsc_type = v.number
			and v.type = 'LR'
			and master.dbo.syslockinfo.req_status = x.number
			and x.type = 'LS'
			and master.dbo.syslockinfo.req_mode + 1 = u.number
			and u.type = 'L'
end

/* now get names of tables and indexes */
declare @DBName sysname
declare @Cmd varchar(4000)

declare DBNameList insensitive cursor  for
select distinct [Database] from #LockLook

Open DBNameList

fetch DBNameList into @DBName
While @@Fetch_Status = 0
begin

	 select @Cmd = 'use ' + @DBName
	 select @Cmd = @Cmd + ' Update #LockLook set TableName = object_name(#LockLook.TableID) '
	 select @Cmd = @Cmd + ' from #LockLook, sysobjects (nolock) '
	 select @Cmd = @Cmd	+ ' where #LockLook.TableID = sysobjects.id '
	 select @Cmd = @Cmd + ' AND #LockLook.[Database] = ''' + @DBName + ''''
	 select @Cmd = @Cmd + ' Update #LockLook Set IndexName = '
	 select @Cmd = @Cmd + ' (select i.name from sysindexes i, sysobjects o'
	 select @Cmd = @Cmd + '  where i.id = #LockLook.TableID and i.indid = #LockLook.IndexID '
	 select @Cmd = @Cmd + '  and #LockLook.TableID = o.id '
	 select @Cmd = @Cmd + '  and #LockLook.[Database] = ''' + @DBName + ''')'
	 select @Cmd = @Cmd + ' from #LockLook, sysobjects '
	 select @Cmd = @Cmd + ' where #LockLook.TableID = sysobjects.id '
	 select @Cmd = @Cmd + ' and #LockLook.[Database] = ''' + @DBName + '''' 

	/* debugging */
	/*select @Cmd */ 
	exec (@Cmd)
	fetch DBNameList into @DBName
end
close DBNameList
deallocate DBNameList


select spid, [Database], TableName, IndexName, Type, Resource, Mode, Status
from #LockLook 
order by spid

return (0) 
